@@ -24,6 +24,7 @@
 #include "OscapScannerRemoteSsh.h"
 #include "ResultViewer.h"
 #include "DiagnosticsDialog.h"
+#include "CommandLineArgsDialog.h"
 #include "TailorProfileDialog.h"
 #include "TailoringWindow.h"
 #include "ScanningSession.h"
@@ -36,7 +37,6 @@
 
 #include <QFileDialog>
 #include <QAbstractEventDispatcher>
-#include <QMessageBox>
 #include <QCloseEvent>
 #include <QDesktopWidget>
 
@@ -51,10 +51,10 @@ extern "C" {
 
 // A dialog to open a tailoring file is displayed after user selects this option
 // from the tailoring combobox.
-const QString TAILORING_CUSTOM_FILE = QObject::tr("(open customization file...)");
+const QString TAILORING_CUSTOM_FILE = QObject::tr("Select customization file...");
 // This option signifies that there is no tailoring being done and the plain
 // content file is used, it also resets tailoring when selected.
-const QString TAILORING_NONE = QObject::tr("(no customization)");
+const QString TAILORING_NONE = QObject::tr("None selected");
 // Signifies that tailoring changes have been made and have not been saved
 // to a file (yet?). Selecting it does nothing.
 const QString TAILORING_UNSAVED = QObject::tr("(unsaved changes)");
@@ -69,6 +69,7 @@ MainWindow::MainWindow(QWidget* parent):
     mQSettings(new QSettings(this)),
 
     mDiagnosticsDialog(0),
+    mCommandLineArgsDialog(0),
 
     mRPMOpenHelper(0),
     mSkipValid(false),
@@ -80,7 +81,9 @@ MainWindow::MainWindow(QWidget* parent):
     mOldTailoringComboBoxIdx(0),
     mLoadedTailoringFileUserData(TAILORING_NO_LOADED_FILE_DATA),
 
-    mIgnoreProfileComboBox(false)
+    mIgnoreProfileComboBox(false),
+
+    mRuleResultsExpanded(false)
 {
     mUI.setupUi(this);
     mUI.progressBar->reset();
@@ -88,6 +91,19 @@ MainWindow::MainWindow(QWidget* parent):
     // we start with localhost which doesn't need remote machine details
     mUI.remoteMachineDetails->hide();
 
+    QObject::connect(
+        mUI.expandRulesButton, SIGNAL(clicked()),
+        this, SLOT(toggleRuleResultsExpanded())
+    );
+
+    QObject::connect(
+        this, SIGNAL(closeMainWindow()),
+        this, SLOT(close()),
+        // Queued to prevent closing the MainWindow before event loop is
+        // entered. Without this the application wouldn't quit gracefully.
+        Qt::QueuedConnection
+    );
+
     QObject::connect(
         this, SIGNAL(showOpenFileDialog()),
         this, SLOT(openFileDialog()),
@@ -148,19 +164,19 @@ MainWindow::MainWindow(QWidget* parent):
 #endif
 
     QObject::connect(
-        mUI.scanButton, SIGNAL(released()),
+        mUI.scanButton, SIGNAL(clicked()),
         this, SLOT(scanAsyncAutoMode())
     );
     QObject::connect(
-        mUI.offlineRemediateButton, SIGNAL(released()),
+        mUI.offlineRemediateButton, SIGNAL(clicked()),
         this, SLOT(offlineRemediateAsync())
     );
     QObject::connect(
-        mUI.cancelButton, SIGNAL(released()),
+        mUI.cancelButton, SIGNAL(clicked()),
         this, SLOT(cancelScanAsync())
     );
     QObject::connect(
-        mUI.clearButton, SIGNAL(released()),
+        mUI.clearButton, SIGNAL(clicked()),
         this, SLOT(clearResults())
     );
 
@@ -179,7 +195,7 @@ MainWindow::MainWindow(QWidget* parent):
 #endif
 
     QObject::connect(
-        mUI.customizeProfileButton, SIGNAL(released()),
+        mUI.customizeProfileButton, SIGNAL(clicked()),
         this, SLOT(customizeProfile())
     );
 
@@ -201,6 +217,9 @@ MainWindow::MainWindow(QWidget* parent):
     mDiagnosticsDialog = new DiagnosticsDialog(this);
     mDiagnosticsDialog->hide();
 
+    mCommandLineArgsDialog = new CommandLineArgsDialog(this);
+    mCommandLineArgsDialog->hide();
+
     QObject::connect(
         mUI.actionShowDiagnostics, SIGNAL(triggered()),
         mDiagnosticsDialog, SLOT(show())
@@ -338,7 +357,7 @@ void MainWindow::openFile(const QString& path)
         setWindowTitle(QObject::tr("SCAP Workbench"));
         mUI.tailoringFileComboBox->clear();
 
-        mDiagnosticsDialog->exceptionMessage(e, QObject::tr("Error while opening file."));
+        mDiagnosticsDialog->exceptionMessage(e, QObject::tr("Error while opening file."), MF_PREFORMATTED_XML);
     }
 }
 
@@ -374,13 +393,8 @@ void MainWindow::openFileDialog()
 
         if (fileOpened())
         {
-            if (QMessageBox::question(this, QObject::tr("Close currently opened file?"),
-                QObject::tr("Opened file has to be closed before '%1' is opened instead.\n\n"
-                            "Are you sure you want to close currently opened file?").arg(path),
-                QMessageBox::Yes | QMessageBox::No, QMessageBox::No) == QMessageBox::Yes)
-            {
+            if (openNewFileQuestionDialog(mScanningSession->getOpenedFilePath()) == QMessageBox::Yes)
                 closeFile();
-            }
             else
                 // user cancelled closing current file, we have to abort
                 break;
@@ -422,29 +436,46 @@ void MainWindow::openSSGDialog(const QString& customDismissLabel)
 
     if (dialog->exec() == QDialog::Accepted)
     {
-        if (fileOpened())
+        if (dialog->loadOtherContentSelected())
         {
-            if (QMessageBox::question(this, QObject::tr("Close currently opened file?"),
-                QObject::tr("Opened file has to be closed before '%1' is opened instead.\n\n"
-                            "Are you sure you want to close currently opened file?").arg(dialog->getSelectedSSGFile()),
-                QMessageBox::Yes | QMessageBox::No, QMessageBox::No) == QMessageBox::Yes)
-            {
-                closeFile();
-            }
-            else
-            {
-                // user cancelled closing current file, we have to abort
-                delete dialog;
-                return;
-            }
+            // don't worry about open files, the openFileDialog will ask if
+            // user wants to close any open file
+            openFileDialogAsync();
         }
+        else
+        {
 
-        openFile(dialog->getSelectedSSGFile());
+            if (fileOpened())
+            {
+                if (openNewFileQuestionDialog(mScanningSession->getOpenedFilePath()) == QMessageBox::Yes)
+                {
+                    closeFile();
+                }
+                else
+                {
+                    // user cancelled closing current file, we have to abort
+                    delete dialog;
+                    return;
+                }
+            }
+            openFile(dialog->getSelectedSSGFile());
+        }
+    }
+    else
+    {
+        // User dissmissed SSGIntegrationDialog
+        if (!fileOpened())
+            closeMainWindowAsync();
     }
 
     delete dialog;
 }
 
+void MainWindow::closeMainWindowAsync()
+{
+    emit closeMainWindow();
+}
+
 void MainWindow::openFileDialogAsync()
 {
     emit showOpenFileDialog();
@@ -535,6 +566,7 @@ void MainWindow::scanAsync(ScannerMode scannerMode)
     const QString target = mUI.localMachineRadioButton->isChecked() ?
         "localhost" : mUI.remoteMachineDetails->getTarget();
 
+    bool fetchRemoteResources = mUI.fetchRemoteResourcesCheckbox->isChecked();
     try
     {
         //if (!mScanner || mScanner->getTarget() != target)
@@ -581,6 +613,7 @@ void MainWindow::scanAsync(ScannerMode scannerMode)
         mScanner->setScanThread(mScanThread);
         mScanner->setMainThread(thread());
         mScanner->setSkipValid(mSkipValid);
+        mScanner->setFetchRemoteResources(fetchRemoteResources);
         mScanner->setSession(mScanningSession);
         mScanner->setScannerMode(scannerMode);
 
@@ -599,6 +632,14 @@ void MainWindow::scanAsync(ScannerMode scannerMode)
 
     mScanner->moveToThread(mScanThread);
 
+    mScanner->setDryRun(mUI.dryRunCheckBox->isChecked());
+    if (mUI.dryRunCheckBox->isChecked())
+    {
+        const QStringList args = mScanner->getCommandLineArgs();
+        mCommandLineArgsDialog->setArgs(args);
+        mCommandLineArgsDialog->show();
+    }
+
     QObject::connect(
         mScanThread, SIGNAL(started()),
         mScanner, SLOT(evaluateExceptionGuard())
@@ -712,6 +753,7 @@ void MainWindow::reloadSession()
 
     mUI.resultViewer->clear();
     mUI.titleLabel->setText(mScanningSession->getBenchmarkTitle());
+    toggleRuleResultsExpanded(false);
     refreshProfiles();
 }
 
@@ -1023,6 +1065,41 @@ void MainWindow::profileComboboxChanged(int index)
     clearResults();
 }
 
+void MainWindow::toggleRuleResultsExpanded()
+{
+    mRuleResultsExpanded = !mRuleResultsExpanded;
+
+    setRuleResultsExpanded(mRuleResultsExpanded);
+}
+
+void MainWindow::toggleRuleResultsExpanded(bool checked)
+{
+    mRuleResultsExpanded = checked;
+
+    setRuleResultsExpanded(mRuleResultsExpanded);
+}
+
+void MainWindow::setRuleResultsExpanded(bool checked)
+{
+    mUI.ruleResultsTree->toggleAllRuleResultDescription(checked);
+    setActionToggleRuleResultsText(checked);
+
+}
+
+void MainWindow::allRuleResultsExpanded(bool checked)
+{
+    mRuleResultsExpanded = checked;
+    setActionToggleRuleResultsText(checked);
+}
+
+void MainWindow::setActionToggleRuleResultsText(bool checked)
+{
+    if(checked)
+        mUI.expandRulesButton->setText(QString("Collapse all"));
+    else
+        mUI.expandRulesButton->setText(QString("Expand all"));
+}
+
 void MainWindow::scanProgressReport(const QString& rule_id, const QString& result)
 {
     /* It is quite hard to accurately estimate completion of SCAP scans.
@@ -1078,6 +1155,12 @@ void MainWindow::scanCanceled()
 void MainWindow::scanFinished()
 {
     scanEnded(false);
+
+    // Clean results if the scan is a dry run
+    // User will see the CommandLineArgsDialog and then the MainWindow
+    // ready for a new scan, or dry run
+    if (mUI.dryRunCheckBox->isChecked())
+        clearResults();
 }
 
 void MainWindow::scanEnded(bool canceled)
@@ -1250,7 +1333,7 @@ void MainWindow::saveTailoring()
 
     try
     {
-        mScanningSession->saveTailoring(path);
+        mScanningSession->saveTailoring(path, true);
         markUnsavedTailoringChanges();
         markLoadedTailoringFile(path);
     }
@@ -1378,26 +1461,26 @@ void MainWindow::about()
 {
     const QString title = "SCAP Workbench " SCAP_WORKBENCH_VERSION;
 
-    const QString versionInfo = QString("SCAP Workbench %1, compiled with Qt %2, using OpenSCAP %3\n\n").arg(SCAP_WORKBENCH_VERSION, QT_VERSION_STR, oscap_get_version());
+    const QString versionInfo = QString("<p>SCAP Workbench %1, compiled with Qt %2, using OpenSCAP %3</p>").arg(SCAP_WORKBENCH_VERSION, QT_VERSION_STR, oscap_get_version());
     const QString description = QObject::tr(
-"This application is called SCAP Workbench, the homepage can be found at \
-<http://fedorahosted.org/scap-workbench>\n\n");
+"<p>This application is called SCAP Workbench, the homepage can be found at \
+<a href='https://www.open-scap.org/tools/scap-workbench/'>https://www.open-scap.org/tools/scap-workbench/</a></p>");
     const QString license = QString(
-"Copyright 2014 Red Hat Inc., Durham, North Carolina.\n\
-All Rights Reserved.\n\
-\n\
-This program is free software: you can redistribute it and/or modify \
+"<p>Copyright 2014 Red Hat Inc., Durham, North Carolina.<br/>\
+All Rights Reserved.</p>\
+\
+<p>This program is free software: you can redistribute it and/or modify \
 it under the terms of the GNU General Public License as published by \
 the Free Software Foundation, either version 3 of the License, or \
-(at your option) any later version.\n\
-\n\
-This program is distributed in the hope that it will be useful, \
+(at your option) any later version.</p>\
+\
+<p>This program is distributed in the hope that it will be useful, \
 but WITHOUT ANY WARRANTY; without even the implied warranty of \
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the \
-GNU General Public License for more details.\n\
-\n\
-You should have received a copy of the GNU General Public License \
-along with this program.  If not, see <http://www.gnu.org/licenses/>.\n\n");
+GNU General Public License for more details.</p>\
+\
+<p>You should have received a copy of the GNU General Public License \
+along with this program.  If not, see <a href='http://www.gnu.org/licenses/'>http://www.gnu.org/licenses/</a>.</p>");
 
     QMessageBox::about(this, title, versionInfo + description + license);
 }
@@ -1406,3 +1489,13 @@ void MainWindow::aboutQt()
 {
     QMessageBox::aboutQt(this);
 }
+
+QMessageBox::StandardButton MainWindow::openNewFileQuestionDialog(const QString& oldFilepath)
+{
+    return QMessageBox::question(this,
+          QObject::tr("Close currently opened file?"),
+          QObject::tr("Opened file '%1' has to be closed before opening another file.\n\n"
+          "Do you want to proceed?").arg(oldFilepath),
+          QMessageBox::Yes | QMessageBox::No, QMessageBox::No
+    );
+}
