@@ -131,17 +131,17 @@ TailoringWindow::TailoringWindow(struct xccdf_policy* policy, struct xccdf_bench
     mUI.setupUi(this);
 
     QObject::connect(
-        mUI.confirmButton, SIGNAL(released()),
+        mUI.confirmButton, SIGNAL(clicked()),
         this, SLOT(confirmAndClose())
     );
 
     QObject::connect(
-        mUI.cancelButton, SIGNAL(released()),
+        mUI.cancelButton, SIGNAL(clicked()),
         this, SLOT(close())
     );
 
     QObject::connect(
-        mUI.deleteProfileButton, SIGNAL(released()),
+        mUI.deleteProfileButton, SIGNAL(clicked()),
         this, SLOT(deleteProfileAndDiscard())
     );
 
@@ -664,11 +664,31 @@ void TailoringWindow::closeEvent(QCloseEvent * event)
 {
     if (!mChangesConfirmed)
     {
-        if (QMessageBox::question(this, QObject::tr("Discard changes?"),
-            mNewProfile ?
-                QObject::tr("Are you sure you want to discard all changes performed in this tailoring window and delete the profile?") :
-                QObject::tr("Are you sure you want to discard all changes performed in this tailoring window?"),
-            QMessageBox::Yes | QMessageBox::No, QMessageBox::No) == QMessageBox::No)
+        QMessageBox msgBox(this);
+        msgBox.setIcon(QMessageBox::Question);
+        msgBox.setStandardButtons(QMessageBox::Yes | QMessageBox::No);
+        if (mNewProfile)
+        {
+            msgBox.setWindowTitle("Confirm delete profile?");
+            msgBox.setText(QObject::tr("Are you sure you want to discard all changes performed in this tailoring window and delete the profile?"));
+            msgBox.button(QMessageBox::Yes)->setText(QObject::tr("Delete profile"));
+            msgBox.button(QMessageBox::No)->setText(QObject::tr("Don't delete profile"));
+        }
+        else
+        {
+            msgBox.setWindowTitle("Confirm discard of changes?");
+            msgBox.setText(QObject::tr("Are you sure you want to discard all changes performed in this tailoring window?"));
+            msgBox.button(QMessageBox::Yes)->setText(QObject::tr("Discard"));
+            msgBox.button(QMessageBox::No)->setText(QObject::tr("Don't discard"));
+        }
+        msgBox.setDefaultButton(QMessageBox::No);
+
+        // Setting dialogbuttonbox-buttons-have-icons to 0 should remove the icons of the
+        // dialog button, but it's not working.
+        // Setting icon-size of the buttons to 0 works for now
+        msgBox.setStyleSheet("QPushButton {icon-size: 0px 0px}");
+
+        if (msgBox.exec() == QMessageBox::No)
         {
             event->ignore();
             return;
@@ -732,7 +752,7 @@ void TailoringWindow::serializeCollapsedItems()
 
 void TailoringWindow::removeOldCollapsedLists()
 {
-    const unsigned int maxAgeInDays = 3 * 31; // ~3 months should be enough for everyone :-P
+    const int maxAgeInDays = 3 * 31; // ~3 months should be enough for everyone :-P
     const QDateTime currentDateTime = QDateTime::currentDateTime();
 
     QStringList keys = mQSettings->childKeys();
@@ -794,9 +814,11 @@ void TailoringWindow::generateValueAffectsRulesMap(struct xccdf_item* item)
         case XCCDF_BENCHMARK:
             items = xccdf_benchmark_get_content(xccdf_item_to_benchmark(item));
             break;
+
         case XCCDF_GROUP:
             items = xccdf_group_get_content(xccdf_item_to_group(item));
             break;
+
         case XCCDF_RULE:
             {
                 struct xccdf_check_iterator* checks = xccdf_rule_get_checks(xccdf_item_to_rule(item));
@@ -823,6 +845,10 @@ void TailoringWindow::generateValueAffectsRulesMap(struct xccdf_item* item)
                 xccdf_check_iterator_free(checks);
             }
             break;
+
+        default:
+            // noop
+            break;
     }
 
     if (items)
@@ -862,7 +888,28 @@ void TailoringWindow::searchNext()
         mSearchSkippedItems = mSearchSkippedItems % matches.size(); // wrap around
 
         QTreeWidgetItem* match = matches.at(mSearchSkippedItems);
-        mUI.itemsTree->setCurrentItem(match);
+
+        if (!match->isDisabled())
+            mUI.itemsTree->setCurrentItem(match);
+        else
+        {
+            // We cannot use setCurrentItem() on disabled items
+            // so we will use a workaround
+
+            QTreeWidgetItem * dummyItem = mUI.itemsTree->currentItem();
+
+            // Setting of "new" current item cause removing selection
+            // from rest of items
+            mUI.itemsTree->setCurrentItem(dummyItem);
+
+            // and we will remove selection from current item, too.
+            dummyItem->setSelected(false);
+
+            // Emulate setting of "currentItem"
+            match->setSelected(true);
+            mUI.itemsTree->scrollToItem(match);
+            emit itemSelectionChanged(match, NULL);
+        }
 
         mSearchBox->setStyleSheet("");
         mSearchFeedback->setText(QObject::tr("Showing match %1 out of %2 total found.").arg(mSearchSkippedItems + 1).arg(matches.size()));
